@@ -177,7 +177,7 @@ void EditorView::onTextOrTitleChanged()
     const auto title = m_model->property("title").toString();
     const auto text = m_model->property("text").toString();
 
-    QRegExp reg("^" + title + "\\s*\\n?" + text + "\\s*$");
+    QRegExp reg("^" + QRegExp::escape(title) + "\\s*\\n?" + QRegExp::escape(text) + "\\s*$");
     if (!reg.exactMatch(m_textEdit->toPlainText()))
         m_textEdit->setPlainText(title + '\n' + text);
 }
