@@ -756,6 +756,37 @@ private slots:
         QCOMPARE(data.value<Domain::Artifact::Ptr>(), provider->data().at(1).staticCast<Domain::Artifact>());
     }
 
+    void shouldMoveOnlyDuringDragAndDrop()
+    {
+        // GIVEN
+        auto queryGenerator = [&] (const QColor &) {
+            return Domain::QueryResult<QColor>::Ptr();
+        };
+        auto flagsFunction = [] (const QColor &) {
+            return Qt::NoItemFlags;
+        };
+        auto dataFunction = [] (const QColor &, int) {
+            return QVariant();
+        };
+        auto setDataFunction = [] (const QColor &, const QVariant &, int) {
+            return false;
+        };
+        auto dropFunction = [] (const QMimeData *, Qt::DropAction, const QColor &) {
+            return false;
+        };
+        auto dragFunction = [] (const QList<QColor> &) {
+            return Q_NULLPTR;
+        };
+
+        Presentation::QueryTreeModel<QColor> model(queryGenerator, flagsFunction,
+                                                   dataFunction, setDataFunction,
+                                                   dropFunction, dragFunction);
+
+        // THEN
+        QCOMPARE(model.supportedDragActions(), Qt::MoveAction);
+        QCOMPARE(model.supportedDropActions(), Qt::MoveAction);
+    }
+
     void shouldCreateMimeData()
     {
         // GIVEN
@@ -812,7 +843,7 @@ private slots:
         QTest::addColumn<bool>("callExpected");
 
         QTest::newRow("drop on object") << -1 << -1 << 2 << true;
-        QTest::newRow("drop between object") << 1 << 0 << -1 << false;
+        QTest::newRow("drop between object") << 1 << 0 << -1 << true;
         QTest::newRow("drop in empty area") << -1 << -1 << -1 << true;
     }
 
@@ -874,6 +905,74 @@ private slots:
             QCOMPARE(colorSeen, parent.data(Presentation::QueryTreeModelBase::ObjectRole).value<QColor>());
         }
     }
+
+    void shouldPreventCyclesByDragAndDrop()
+    {
+        // GIVEN
+        bool dropCalled = false;
+
+        auto topProvider = Domain::QueryResultProvider<QString>::Ptr::create();
+        topProvider->append("1");
+        topProvider->append("2");
+        topProvider->append("3");
+
+        auto firstLevelProvider = Domain::QueryResultProvider<QString>::Ptr::create();
+        firstLevelProvider->append("2.1");
+        firstLevelProvider->append("2.2");
+        firstLevelProvider->append("2.3");
+
+        auto secondLevelProvider = Domain::QueryResultProvider<QString>::Ptr::create();
+        secondLevelProvider->append("2.1.1");
+        secondLevelProvider->append("2.1.2");
+        secondLevelProvider->append("2.1.3");
+
+        auto queryGenerator = [&] (const QString &string) {
+            if (string.isEmpty())
+                return Domain::QueryResult<QString>::create(topProvider);
+            else if (string == "2")
+                return Domain::QueryResult<QString>::create(firstLevelProvider);
+            else if (string == "2.1")
+                return Domain::QueryResult<QString>::create(secondLevelProvider);
+            else
+                return Domain::QueryResult<QString>::Ptr();
+        };
+        auto flagsFunction = [] (const QString &) {
+            return Qt::NoItemFlags;
+        };
+        auto dataFunction = [] (const QString &, int) {
+            return QVariant();
+        };
+        auto setDataFunction = [] (const QString &, const QVariant &, int) {
+            return false;
+        };
+        auto dropFunction = [&] (const QMimeData *, Qt::DropAction, const QString &) {
+            dropCalled = true;
+            return false;
+        };
+        auto dragFunction = [] (const QStringList &strings) -> QMimeData* {
+            auto data = new QMimeData;
+            data->setData("application/x-zanshin-object", "object");
+            data->setProperty("objects", QVariant::fromValue(strings));
+            return data;
+        };
+
+        Presentation::QueryTreeModel<QString> model(queryGenerator, flagsFunction,
+                                                    dataFunction, setDataFunction,
+                                                    dropFunction, dragFunction);
+        new ModelTest(&model);
+
+        const auto indexes = QModelIndexList() << model.index(0, 0)
+                                               << model.index(1, 0)
+                                               << model.index(1, 0, model.index(1, 0));
+
+        // WHEN
+        auto data = model.mimeData(indexes);
+        const auto parent = model.index(1, 0, model.index(0, 0, model.index(1, 0)));
+        model.dropMimeData(data, Qt::MoveAction, -1, -1, parent);
+
+        // THEN
+        QVERIFY(!dropCalled);
+    }
 };
 
 QTEST_MAIN(QueryTreeModelTest)
